@@ -18,8 +18,9 @@ sub doInventory {
     my (%params) = @_;
 
     my $inventory = $params{inventory};
+    my $logger    = $params{logger};
 
-    my $infos = getSystemProfilerInfos();
+    my $infos = getSystemProfilerInfos(logger => $logger);
     my $info = $infos->{Printers};
 
     foreach my $printer (keys %$info) {
