@@ -21,7 +21,9 @@ our @EXPORT = qw(
     getNetworkMaskIPv6
     hex2canonical
     alt2canonical
-    resolv
+    resolve
+    compile
+    isPartOf
 );
 
 my $dec_byte        = qr/[0-9]{1,3}/;
@@ -37,18 +39,18 @@ our $ip_address_pattern = qr/
 /x;
 
 our $alt_mac_address_pattern = qr/
-    $padded_hex_byte 
-    $padded_hex_byte 
-    $padded_hex_byte 
-    $padded_hex_byte 
-    $padded_hex_byte 
+    $padded_hex_byte
+    $padded_hex_byte
+    $padded_hex_byte
+    $padded_hex_byte
+    $padded_hex_byte
     $padded_hex_byte
 /x;
 
 our $hex_ip_address_pattern = qr/
-    $padded_hex_byte 
-    $padded_hex_byte 
-    $padded_hex_byte 
+    $padded_hex_byte
+    $padded_hex_byte
+    $padded_hex_byte
     $padded_hex_byte
 /x;
 
@@ -112,7 +114,7 @@ sub getNetworkMaskIPv6 {
     return ip_compress_address(ip_bintoip(ip_get_mask($prefix, 6), 6), 6);
 }
 
-sub resolv {
+sub resolve {
     my ($string, $logger) = @_;
 
     my @ret;
@@ -125,7 +127,7 @@ sub resolv {
     );
     if ($error && $logger) {
         $logger->error("unable to resolve `$string': $error");
-        next;
+        return;
     }
 
     # and push all of their addresses in the list
@@ -135,17 +137,61 @@ sub resolv {
             Socket::GetAddrInfo::NI_NUMERICHOST(),
         );
         # Drop the zone index. Net::IP do not support them
-        $host =~ s/%.*$//;
         if ($error && $logger) {
-            $logger->error("unable to get host `$host' IP address: $error");
+            $logger->error("unable to get hostname of IP address `$result->{addr}': $error");
             next;
         }
+        $host =~ s/%.*$//;
         push @ret, Net::IP->new($host);
     }
 
     return @ret;
 }
 
+sub compile {
+    my ($string, $logger) = @_;
+
+    return unless $string;
+
+    # that's already an IP address, just convert it
+    return Net::IP->new($string)
+        if $string =~ /^$ip_address_pattern/;
+
+    # otherwise resolve the name
+    return resolve($string, $logger);
+}
+
+sub isPartOf {
+    my ($string, $ranges, $logger) = @_;
+
+    return unless $string;
+    return unless $ranges;
+
+    my $address = Net::IP->new($string);
+
+    if (!$address) {
+        $logger->error("Not well formatted source IP: $string");
+        return;
+    }
+
+    foreach my $range (@{$ranges}) {
+        my $result = $address->overlaps($range);
+
+        if (!$result && Net::IP::Error()) {
+            $logger->debug("Server: ".Net::IP::Error());
+            next;
+        }
+
+        # included in trusted range
+        return 1 if $result == $IP_A_IN_B_OVERLAP;
+
+        # equals trusted address
+        return 1 if $result == $IP_IDENTICAL;
+    }
+
+    return 0;
+}
+
 1;
 __END__
 
@@ -201,6 +247,17 @@ Returns the network mask for IPv4.
 
 Returns the network mask for IPv6.
 
-=head2 resolv($string)
+=head2 resolve($host, $logger)
 
-Returns an array of Net::IP for the given $string
+Returns a list of addresses, as Net::IP objects, for the given host name, as a
+string.
+
+=head2 compile($spec, $logger)
+
+Returns a list of addresses, as Net::IP objects, for the given IP address or
+host name, as a string.
+
+=head2 isPartOf($address, $addresses, $logger)
+
+Returns true if the given address, as a string, is part of any address from the
+given list, as Net::IP objects.
