@@ -8,7 +8,6 @@ use XML::TreePP;
 sub new {
     my ($class, %params) = @_;
 
-    die "no deviceid parameter" unless $params{deviceid};
     die "no query parameter" unless $params{query};
 
     my $self = {};
@@ -24,6 +23,7 @@ sub getContent {
     my ($self) = @_;
 
     my $tpp = XML::TreePP->new(indent => 2);
+
     return $tpp->write({ REQUEST => $self->{h} });
 }
 
@@ -56,11 +56,10 @@ the logger object to use
 
 =item I<deviceid>
 
-the agent identifier (mandatory)
+the agent identifier (optional)
 
 =back
 
 =head2 getContent
 
 Get XML content.
-
